/*  OpenTally: Open-source election vote counting
 *  Copyright © 2021 Lee Yingtong Li (RunasSudo)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

function clickAdvancedOptions() {
	if (document.getElementById('divAdvancedOptions').style.display === 'none') {
		document.getElementById('divAdvancedOptions').style.display = 'grid';
		document.getElementById('btnAdvancedOptions').innerHTML = 'Hide advanced options';
	} else {
		document.getElementById('divAdvancedOptions').style.display = 'none';
		document.getElementById('btnAdvancedOptions').innerHTML = 'Show advanced options';
	}
}

var tblResult = document.getElementById('result');
var divLogs2 = document.getElementById('resultLogs2');
var olStageComments;

var worker = new Worker('worker.js');

worker.onmessage = function(evt) {
	if (evt.data.type === 'init') {
		document.getElementById('spanRevNum').innerText = evt.data.version;
		document.getElementById('divLoading').style.display = 'none';
		document.getElementById('divUI').style.display = 'block';
		
		// Init dropdowns
		// Can't compute correct width until #divUI, etc. is display: block
		//document.getElementById('divAdvancedOptions').style.display = 'grid';
		//for (let elSel of document.querySelectorAll('select')) {
		let elSel = document.getElementById('selPreset'); {
			var sel = new CustomSelect({elem: elSel});
			sel.open();
			document.getElementById('custom-' + elSel.id).style.width = (document.getElementById('custom-' + elSel.id).querySelector('.js-Dropdown-list').clientWidth + 32) + 'px';
			sel.close();
		}
		//document.getElementById('divAdvancedOptions').style.display = 'none';
	} else if (evt.data.type === 'initResultsTable') {
		tblResult.innerHTML = evt.data.content;
		divLogs2.innerHTML = '<p>Stage comments:</p>';
		olStageComments = document.createElement('ol');
		divLogs2.append(olStageComments);
		
	} else if (evt.data.type === 'describeCount') {
		document.getElementById('resultLogs1').innerHTML = evt.data.content;
		
	} else if (evt.data.type === 'updateResultsTable') {
		for (let row = 0; row < evt.data.result.length; row++) {
			if (evt.data.result[row]) {
				tblResult.rows[row].insertAdjacentHTML('beforeend', evt.data.result[row]);
				
				// Update candidate status
				if (
					(document.getElementById('selReport').value == 'votes' && row >= 3 && row % 2 == 1) ||
					(document.getElementById('selReport').value == 'ballots_votes' && row >= 4 && row % 2 == 0)
				) {
					if (tblResult.rows[row].lastElementChild.classList.contains('elected')) {
						tblResult.rows[row].cells[0].classList.add('elected');
					} else {
						tblResult.rows[row].cells[0].classList.remove('elected');
					}
				}
			}
		}
		
	} else if (evt.data.type === 'updateStageComments') {
		let elLi = document.createElement('li');
		elLi.id = 'stage' + (olStageComments.childElementCount + 1);
		elLi.innerHTML = evt.data.comment;
		olStageComments.append(elLi);
		
	} else if (evt.data.type === 'finalResultSummary') {
		divLogs2.insertAdjacentHTML('beforeend', evt.data.summary);
		document.getElementById('printPane').style.display = 'block';
		
		// Linkify stage numbers
		document.querySelectorAll('tr.stage-no a').forEach(function(elA) {
			elA.onclick = function() {
				olStageComments.childNodes.forEach(function(elLi) { elLi.classList.remove('highlight'); });
				document.getElementById(elA.href.substring(elA.href.indexOf('#') + 1)).classList.add('highlight');
			};
		});
		
	} else if (evt.data.type === 'requireInput') {
		let response = window.prompt(evt.data.message);
		while (response === null) {
			response = window.prompt(evt.data.message);
		}
		worker.postMessage({'type': 'userInput', 'response': response});
		
	} else if (evt.data.type === 'errorMessage') {
		divLogs2.insertAdjacentHTML('beforeend', evt.data.message);
	}
}

worker.onerror = function(evt) {
	alert('An unknown error occurred while counting the votes. More details may be available in the browser\'s developer console.');
}

async function clickCount() {
	if (document.getElementById('bltFile').files.length === 0) {
		return;
	}
	
	// Read BLT file
	let bltPath = document.getElementById('bltFile').value;
	bltPath = bltPath.substring(Math.max(bltPath.lastIndexOf('\\'), bltPath.lastIndexOf('/')) + 1);
	
	let bltFile = document.getElementById('bltFile').files[0];
	let bltData = await bltFile.text();
	
	// Read CON file (if applicable)
	let conPath = null;
	let conData = null;
	if (document.getElementById('conFile').files.length > 0) {
		conPath = document.getElementById('conFile').value;
		conPath = conPath.substring(Math.max(conPath.lastIndexOf('\\'), conPath.lastIndexOf('/')) + 1);
		
		let conFile = document.getElementById('conFile').files[0];
		conData = await conFile.text();
	}
	
	// Init STV options
	let optsStr = [
		document.getElementById('chkRoundSFs').checked ? parseInt(document.getElementById('txtRoundSFs').value) : null,
		document.getElementById('chkRoundValues').checked ? parseInt(document.getElementById('txtRoundValues').value) : null,
		document.getElementById('chkRoundVotes').checked ? parseInt(document.getElementById('txtRoundVotes').value) : null,
		document.getElementById('chkRoundQuota').checked ? parseInt(document.getElementById('txtRoundQuota').value) : null,
		document.getElementById('selSumTransfers').value,
		document.getElementById('txtMeekSurplusTolerance').value,
		document.getElementById('chkNormaliseBallots').checked,
		document.getElementById('selQuota').value,
		document.getElementById('selQuotaCriterion').value,
		document.getElementById('selQuotaMode').value,
		document.getElementById('selTies').value.split(','),
		document.getElementById('txtSeed').value,
		document.getElementById('selMethod').value,
		document.getElementById('selSurplus').value,
		document.getElementById('selPapers').value == 'transferable',
		document.getElementById('selExclusion').value,
		document.getElementById('chkMeekNZExclusion').checked,
		document.getElementById('selSample').value,
		document.getElementById('chkSamplePerBallot').checked,
		document.getElementById('chkBulkElection').checked,
		document.getElementById('chkBulkExclusion').checked,
		document.getElementById('chkDeferSurpluses').checked,
		document.getElementById('chkImmediateElect').checked,
		document.getElementById('txtMinThreshold').value,
		conPath,
		"guard_doom",
		parseInt(document.getElementById('txtPPDP').value),
	];
	
	// Reset UI
	document.getElementById('printPane').style.display = 'none';
	document.getElementById('resultLogs1').innerHTML = '';
	tblResult.innerHTML = '';
	divLogs2.innerHTML = '';
	
	// Dispatch to worker
	worker.postMessage({
		'type': 'countElection',
		// Data
		'bltData': bltData,
		'conData': conData,
		'bltPath': bltPath,
		'conPath': conPath,
		// Options
		'optsStr': optsStr,
		'numbers': document.getElementById('selNumbers').value,
		'decimals': document.getElementById('txtDP').value,
		'normaliseBallots': document.getElementById('chkNormaliseBallots').checked,
		'reportStyle': document.getElementById('selReport').value,
	});
}

// Provide a default seed
if (document.getElementById('txtSeed').value === '') {
	function pad(x) { if (x < 10) { return '0' + x; } return '' + x; }
	let d = new Date();
	document.getElementById('txtSeed').value = d.getFullYear() + pad(d.getMonth() + 1) + pad(d.getDate());
}

// Print logic

async function printResult() {
	olStageComments.childNodes.forEach(function(elLi) { elLi.classList.remove('highlight'); });
	
	let printableWidth; // Printable width in CSS pixels
	let paperSize = document.getElementById('selPaperSize').value;
	if (paperSize === 'A4') {
		printableWidth = (29.7 - 2) * 96 / 2.54;
	} else if (paperSize === 'A3') {
		printableWidth = (42.0 - 2) * 96 / 2.54;
	} else if (paperSize === 'letter') {
		printableWidth = (27.9 - 2) * 96 / 2.54;
	}
	printableWidth = Math.round(printableWidth);
	
	let wprint = window.open('');
	wprint.document.title = 'OpenTally Report';
	
	// Add stylesheets
	let numToLoad = 0;
	let numLoaded = -1;
	
	function onLoadStylesheet() {
		numLoaded++;
		if (numLoaded == numToLoad) {
			wprint.print();
		}
	}
	
	for (let elCSSBase of document.querySelectorAll('head link')) {
		numToLoad++;
		let elCSS = wprint.document.createElement('link');
		elCSS.rel = elCSSBase.rel;
		elCSS.type = elCSSBase.type;
		if (elCSSBase.href.endsWith('?v=GITVERSION')) {
			elCSS.href = elCSSBase.href.replace('?v=GITVERSION', '?v=' + Math.random());
		} else {
			elCSS.href = elCSSBase.href;
		}
		
		elCSS.onload = onLoadStylesheet;
		wprint.document.head.appendChild(elCSS);
	}
	
	// Configure printing
	let elStyle = wprint.document.createElement('style');
	elStyle.innerHTML = '@page { size: ' + paperSize + ' landscape; margin: 1cm; } @media print { body { padding: 0; } }';
	wprint.document.head.appendChild(elStyle);
	
	let elContainer = wprint.document.createElement('div');
	elContainer.id = 'printContainer';
	elContainer.style.width = printableWidth + 'px';
	wprint.document.body.appendChild(elContainer);
	
	// Copy result logs 1
	let divResultLogs1 = document.getElementById('resultLogs1');
	let divResultLogs2 = wprint.document.createElement('div');
	divResultLogs2.innerHTML = divResultLogs1.innerHTML;
	elContainer.appendChild(divResultLogs2);
	
	// Parse table, accounting for colspan/rowspan
	let elTrs1 = document.querySelector('#result').rows;
	let rows = [];
	for (let elTr1 of elTrs1) {
		rows.push([]);
	}
	for (let r = 0; r < elTrs1.length; r++) {
		for (let c = 0; c < elTrs1[r].cells.length; c++) {
			let elTd1 = elTrs1[r].cells[c];
			rows[r].push(elTd1);
			
			let colspan = elTd1.getAttribute('colspan');
			if (colspan !== null) {
				colspan = parseInt(colspan);
				// Add ghost cells
				for (let i = 1; i < colspan; i++) {
					rows[r].push(null);
				}
			}
			
			let rowspan = elTd1.getAttribute('rowspan');
			// NB: Only works for rowspan in first column
			if (rowspan !== null && c == 0) {
				rowspan = parseInt(rowspan);
				// Add ghost cells
				for (let i = 1; i < rowspan; i++) {
					rows[r + i].push(null);
				}
			}
		}
	}
	
	function copyColumn(c, elTrs2) {
		let tdsAdded = [];
		for (let r = 0; r < rows.length; r++) {
			if (c < rows[r].length) {
				let elTd1 = rows[r][c];
				if (elTd1 !== null) {
					let elTd2 = wprint.document.createElement('td');
					elTd2.innerHTML = elTd1.innerHTML;
					elTd2.className = elTd1.className;
					if (elTd1.getAttribute('rowspan') !== null) { elTd2.setAttribute('rowspan', elTd1.getAttribute('rowspan')); }
					if (elTd1.getAttribute('colspan') !== null) { elTd2.setAttribute('colspan', elTd1.getAttribute('colspan')); }
					if (elTd1.getAttribute('style') !== null) { elTd2.setAttribute('style', elTd1.getAttribute('style')); }
					elTrs2[r].appendChild(elTd2);
					tdsAdded.push(elTd2);
				}
			}
		}
		return tdsAdded;
	}
	
	async function copyTableColumns(startCol) {
		let modelRow = document.getElementById('selReport').value === 'ballots_votes' ? rows[4] : rows[3];
		
		// Add table
		let elTable2 = wprint.document.createElement('table');
		elTable2.className = 'result';
		if (startCol > 1) {
			elTable2.style.pageBreakBefore = 'always';
		}
		elContainer.appendChild(elTable2);
		
		// Add rows
		let elTrs2 = [];
		for (let elTr1 of elTrs1) {
			let elTr2 = wprint.document.createElement('tr');
			elTr2.className = elTr1.className;
			elTrs2.push(elTr2);
			elTable2.appendChild(elTr2);
		}
		
		// Copy first column
		copyColumn(0, elTrs2);
		
		// How many columns to copy?
		let totalWidth = modelRow[0].clientWidth;
		let endCol;
		for (endCol = startCol; endCol < modelRow.length; ) {
			// Check first column
			if (totalWidth + modelRow[endCol].clientWidth > printableWidth) {
				break;
			}
			
			if (document.getElementById('selReport').value === 'ballots_votes' && endCol + 1 < modelRow.length) {
				// Check second column
				if (totalWidth + modelRow[endCol].clientWidth + modelRow[endCol + 1].clientWidth > printableWidth) {
					break;
				}
			}
			
			// Ok!
			totalWidth += modelRow[endCol].clientWidth;
			endCol++;
			
			if (document.getElementById('selReport').value === 'ballots_votes' && endCol < modelRow.length) {
				// Second column
				totalWidth += modelRow[endCol].clientWidth;
				endCol++;
			}
		}
		
		// Copy columns
		let stages = [];
		for (let c = startCol; c < endCol; c++) {
			if (rows[0][c] !== null && rows[0][c].querySelector('a')) {
				// Track stage headings copied
				stages.push(parseInt(rows[0][c].querySelector('a').innerHTML));
			}
			copyColumn(c, elTrs2);
		}
		
		// Copy stage comments
		elContainer.insertAdjacentHTML('beforeend', '<p>Stage comments:</p>');
		let olStageComments2 = wprint.document.createElement('ol');
		olStageComments2.start = stages[0];
		elContainer.append(olStageComments2);
		for (let stage of stages) {
			olStageComments2.insertAdjacentHTML('beforeend', olStageComments.children[stage-1].outerHTML);
		}
		
		if (endCol < modelRow.length) {
			// Start new table if columns remain
			copyTableColumns(endCol);
		} else {
			// Copy winning candidates
			elContainer.insertAdjacentHTML('beforeend', '<p>Count complete. The winning candidates are, in order of election:</p>');
			elContainer.insertAdjacentHTML('beforeend', divLogs2.lastElementChild.outerHTML);
		}
	}
	
	// Adjust results table to width
	document.getElementById('resultsDiv').style.width = printableWidth + 'px';
	await new Promise(window.requestAnimationFrame); // Allow DOM to update
	
	// Copy table
	await copyTableColumns(1);
	
	// Restore original view
	document.getElementById('resultsDiv').style.width = 'auto';
	
	// Trigger print when ready
	onLoadStylesheet();
}

// Presets

function changePreset() {
	if (document.getElementById('selPreset').value === 'wigm') {
		document.getElementById('selQuotaCriterion').value = 'gt';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'rational';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = false;
		document.getElementById('chkRoundVotes').checked = false;
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'wig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'scottish') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '5';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = false;
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '5';
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'per_ballot';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'wig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'meek87') {
		document.getElementById('selQuotaCriterion').value = 'gt';
		document.getElementById('selQuota').value = 'droop_exact';
		document.getElementById('selQuotaMode').value = 'dynamic_by_total';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = false;
		document.getElementById('chkMeekNZExclusion').checked = false;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = false;
		document.getElementById('chkRoundVotes').checked = false;
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		//document.getElementById('selSumTransfers').value = 'single_step';
		document.getElementById('txtMeekSurplusTolerance').value = '0.001%';
		//document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'meek';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'meek06') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'dynamic_by_total';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('chkMeekNZExclusion').checked = false;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '12';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '9';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '9';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '9';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '9';
		//document.getElementById('selSumTransfers').value = 'single_step';
		document.getElementById('txtMeekSurplusTolerance').value = '0.0001';
		//document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'meek';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'forwards,random';
	} else if (document.getElementById('selPreset').value === 'meeknz') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'dynamic_by_total';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('chkMeekNZExclusion').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '12';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '9';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '9';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '9';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '9';
		//document.getElementById('selSumTransfers').value = 'single_step';
		document.getElementById('txtMeekSurplusTolerance').value = '0.0001';
		//document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'meek';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'forwards,random';
	} else if (document.getElementById('selPreset').value === 'senate') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false; // Senate "bulk exclusion" does not permit quota to be exceeded
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'rational';
		document.getElementById('txtPPDP').value = '0';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '0';
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_order';
		document.getElementById('selMethod').value = 'uig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'wa') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'rational';
		document.getElementById('txtPPDP').value = '0';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '0';
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_order';
		document.getElementById('selMethod').value = 'wig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'by_source';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'act') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'rational';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '6';
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_order';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'minneapolis') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = true;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = false;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '4';
		document.getElementById('txtPPDP').value = '4';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = false;
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '4';
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'wig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'random';
	} else if (document.getElementById('selPreset').value === 'cambridge') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('selSample').value = 'nth_ballot';
		document.getElementById('chkSamplePerBallot').checked = true;
		document.getElementById('txtMinThreshold').value = '49';
		document.getElementById('selNumbers').value = 'rational';
		document.getElementById('txtPPDP').value = '0';
		document.getElementById('chkNormaliseBallots').checked = true;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selMethod').value = 'cincinnati';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'single_stage';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'wright') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = true;
		document.getElementById('chkDeferSurpluses').checked = false;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '0';
		document.getElementById('chkRoundVotes').checked = false;
		document.getElementById('chkRoundSFs').checked = false;
		document.getElementById('chkRoundValues').checked = false;
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'wig';
		document.getElementById('selPapers').value = 'both';
		document.getElementById('selExclusion').value = 'wright';
		document.getElementById('selTies').value = 'random';
	} else if (document.getElementById('selPreset').value === 'prsa77') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '6';
		document.getElementById('txtPPDP').value = '3';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '3';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '3';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '3';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '3';
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_order';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'parcels_by_order';
		document.getElementById('selTies').value = 'backwards,random';
	} else if (document.getElementById('selPreset').value === 'ers97') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop_exact';
		document.getElementById('selQuotaMode').value = 'ers97';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = true;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '2';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '2';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '2';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '2';
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'forwards,random';
	} else if (document.getElementById('selPreset').value === 'ers76') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop_exact';
		document.getElementById('selQuotaMode').value = 'ers76';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = true;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '2';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '2';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '2';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '2';
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'forwards,random';
	} else if (document.getElementById('selPreset').value === 'ers73') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop_exact';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = true;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '2';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '2';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '2';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '2';
		document.getElementById('selSumTransfers').value = 'by_value';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'forwards,random';
	} else if (document.getElementById('selPreset').value === 'cofe') {
		document.getElementById('selQuotaCriterion').value = 'geq';
		document.getElementById('selQuota').value = 'droop';
		document.getElementById('selQuotaMode').value = 'static';
		document.getElementById('chkBulkElection').checked = true;
		document.getElementById('chkBulkExclusion').checked = false;
		document.getElementById('chkDeferSurpluses').checked = true;
		document.getElementById('chkImmediateElect').checked = true;
		document.getElementById('txtMinThreshold').value = '0';
		document.getElementById('selNumbers').value = 'fixed';
		document.getElementById('txtDP').value = '5';
		document.getElementById('txtPPDP').value = '2';
		document.getElementById('chkNormaliseBallots').checked = false;
		document.getElementById('chkRoundQuota').checked = true;
		document.getElementById('txtRoundQuota').value = '2';
		document.getElementById('chkRoundVotes').checked = true;
		document.getElementById('txtRoundVotes').value = '2';
		document.getElementById('chkRoundSFs').checked = true;
		document.getElementById('txtRoundSFs').value = '2';
		document.getElementById('chkRoundValues').checked = true;
		document.getElementById('txtRoundValues').value = '2';
		document.getElementById('selSumTransfers').value = 'per_ballot';
		document.getElementById('selSurplus').value = 'by_size';
		document.getElementById('selMethod').value = 'eg';
		document.getElementById('selPapers').value = 'transferable';
		document.getElementById('selExclusion').value = 'by_value';
		document.getElementById('selTies').value = 'forwards,random';
	}
}
